/**
 * The Abiquo Platform
 * Cloud management application for hybrid clouds
 * Copyright (C) 2008 - Abiquo Holdings S.L.
 *
 * This application is free software; you can redistribute it and/or
 * modify it under the terms of the GNU LESSER GENERAL PUBLIC
 * LICENSE as published by the Free Software Foundation under
 * version 3 of the License
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * LESSER GENERAL PUBLIC LICENSE v.3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
package com.abiquo.ssm.plugin;

import java.util.List;

import com.abiquo.ssm.exception.PluginException;
import com.abiquo.ssm.model.Device;
import com.abiquo.ssm.model.IscsiAddress;
import com.abiquo.ssm.model.Pool;
import com.abiquo.ssm.model.Volume;
import com.abiquo.ssm.model.metadata.StorageOp;
import com.abiquo.ssm.model.metadata.StoragePluginMetadata;
import com.abiquo.ssm.plugin.annotations.Provides;

/**
 * Defines an storage plugin that performs the operations in the target storage devices.
 * 
 * @author Ignasi Barrera
 */
public interface StoragePlugin
{
    // Plugin metadata

    /**
     * Gets the information describing the features implemented by the plugin.
     * 
     * @return The plugin metadata.
     * @throws PluginException If the metadata for the plugin cannot be obtained.
     */
    public StoragePluginMetadata getPluginMetadata() throws PluginException;

    // Pool operations

    /**
     * Get all the pools defined in the given device.
     * <p>
     * The storage pools are the containers for the volumes that will be created by Abiquo.
     * 
     * @param device The storage device where to find the pools.
     * @return The pools defined in the given device.
     * @throws PluginException If an error occurs.
     */
    @Provides(StorageOp.POOL_ENUMERATE)
    public List<Pool> listPools(Device device) throws PluginException;

    /**
     * Get the information of the given pool.
     * <p>
     * The storage pools are the containers for the volumes that will be created by Abiquo.
     * 
     * @param device The storage device where to find the pool.
     * @param poolName The name of the pool to retrieve.
     * @return The requested pool.
     */
    @Provides(StorageOp.POOL_GET_DETAILS)
    public Pool getPool(Device device, String poolName) throws PluginException;

    // Volume operations

    /**
     * Gets all the volumes in the given pool.
     * <p>
     * Volumes are block devices that will be attached to the virtual machines using the iSCSI
     * protocol.
     * 
     * @param device The device where the volumes are.
     * @param pool The pool where the volumes are.
     * @return The list of volumes in the pool.
     */
    @Provides(StorageOp.VOLUME_ENUMERATE)
    public List<Volume> listVolumes(Device device, Pool pool) throws PluginException;

    /**
     * Gets the information of the given volume.
     * <p>
     * Volumes are block devices that will be attached to the virtual machines using the iSCSI
     * protocol.
     * 
     * @param device The device where the volumes is.
     * @param pool The pool where the volume is.
     * @param uuid The uuid of the volume to retrieve.
     * @return The information of the given volume.
     */
    @Provides(StorageOp.VOLUME_GET_DETAILS)
    public Volume getVolume(Device device, Pool pool, final String uuid) throws PluginException;

    /**
     * Creates a new volume.
     * <p>
     * Volumes are block devices that will be attached to the virtual machines using the iSCSI
     * protocol.
     * 
     * @param device The device where the volumes will be created.
     * @param pool The pool where the volume will be created.
     * @param volume The volume information.
     * @return The information of the created volume.
     */
    @Provides(StorageOp.VOLUME_CREATE)
    public Volume createVolume(Device device, Pool pool, Volume volume) throws PluginException;

    /**
     * Deletes the given volume.
     * <p>
     * Volumes are block devices that will be attached to the virtual machines using the iSCSI
     * protocol.
     * 
     * @param device The device where the volumes is.
     * @param pool The pool where the volume is.
     * @param uuid The uuid of the volume to delete.
     */
    @Provides(StorageOp.VOLUME_DELETE)
    public void deleteVolume(Device device, Pool pool, final String uuid) throws PluginException;

    /**
     * Updates an existing volume.
     * <p>
     * Volumes are block devices that will be attached to the virtual machines using the iSCSI
     * protocol.
     * 
     * @param device The device where the volumes is.
     * @param pool The pool where the volume is.
     * @param volume The new volume information.
     * @return The updated volume.
     */
    @Provides(StorageOp.VOLUME_RESIZE)
    public Volume resizeVolume(Device device, Pool pool, Volume volume) throws PluginException;

    // Initiator operations

    /**
     * Allow the provided initiator to access the given volume.
     * <p>
     * This method is used to grant access to the given iSCSI initiator to the volume identified by
     * the given UUID.
     * 
     * @param device The device where the volumes is.
     * @param pool The pool where the volume is.
     * @param ostype, the operating system type of the <code>initiatorIQN</code> (if null or empty
     *            set to <code>default</code>)
     * @param initiatorIQN, client initiator IQN.
     * @param volumeUuid The uuid of the volume.
     * @return The iSCSI address where the provided client initiator can reach the volume.
     */
    @Provides(StorageOp.INITIATOR_GRANT)
    public IscsiAddress grantAccess(Device device, Pool pool, String ostype,
        final String initiatorIQN, final String volumeUuid) throws PluginException;

    /**
     * Deny access for the provided initiator to the given volume.
     * <p>
     * This method is used to revoke access for the given iSCSI initiator.
     * 
     * @param device The device where the volumes is.
     * @param pool The pool where the volume is.
     * @param ostype, the operating system type of the <code>initiatorIQN</code> (if null or empty
     *            set to <code>default</code>)
     * @param initiatorIQN, client initiator IQN.
     * @param volumeUuid The uuid of the volume.
     */
    @Provides(StorageOp.INITIATOR_REVOKE)
    public void revokeAccess(Device device, Pool pool, String ostype, final String initiatorIQN,
        final String volumeUuid) throws PluginException;
}
