/**
 * The Abiquo Platform
 * Cloud management application for hybrid clouds
 * Copyright (C) 2008 - Abiquo Holdings S.L.
 *
 * This application is free software; you can redistribute it and/or
 * modify it under the terms of the GNU LESSER GENERAL PUBLIC
 * LICENSE as published by the Free Software Foundation under
 * version 3 of the License
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * LESSER GENERAL PUBLIC LICENSE v.3 for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
package com.abiquo.ssm.plugin.remote;

import java.util.List;

import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;

import com.abiquo.ssm.model.IscsiAddress;
import com.abiquo.ssm.model.Pool;
import com.abiquo.ssm.model.Volume;

/**
 * Facade for the REST interface of the storage agents.
 * 
 * @author vmahe
 */
@Path(RemoteAgentFacade.POOLS_PATH)
public interface RemoteAgentFacade
{

    /** The pools resource path. */
    public static final String POOLS_PATH = "pools";

    /** The pool resource parameter. */
    public static final String POOL_PARAM = "pool";

    /** The pool resource path. */
    public static final String POOL_PATH = "{" + POOL_PARAM + "}";

    /** The volumes resource path. */
    public static final String VOLUMES_PATH = POOL_PATH + "/" + "volumes";

    /** The volume parameter. */
    public static final String VOLUME_PARAM = "uuid";

    /** The single volume resource path. */
    public static final String VOLUME_PATH = VOLUMES_PATH + "/" + "{" + VOLUME_PARAM + "}";

    /** The operating system type resource parameter. */
    public static final String OSTYPE_PARAM = "ostype";

    /** The operating system type resource parameter. */
    public static final String INITIATORS_PATH = POOL_PATH + "/initiators/{" + OSTYPE_PARAM + "}";

    /** The initiators resource path. */
    public static final String INITIATOR_PATH = INITIATORS_PATH + "/{" + VOLUME_PARAM + "}";

    /** The initiator IQN parameter. */
    public static final String INITIATOR_PARAM = "initiatorIQN";

    // Pool methods

    /**
     * Get all the pools defined in the target device.
     * 
     * @return The pools defined in the target device.
     */
    @GET
    public List<Pool> getAllPools();

    /**
     * Get the information of the given pool.
     * 
     * @param poolName The name of the pool to retrieve.
     * @return The requested pool.
     */
    @GET
    @Path(POOL_PATH)
    public Pool getPool(@PathParam(POOL_PARAM) String poolName);

    // Volume methods

    /**
     * Gets all the volumes in the given pool.
     * 
     * @param poolName The pool where the volumes are.
     * @return The list of volumes in the pool.
     */
    @GET
    @Path(VOLUMES_PATH)
    public List<Volume> getAllVolumes(@PathParam(POOL_PARAM) String poolName);

    /**
     * Gets the information of the given volume.
     * 
     * @param poolName The pool where the volume is.
     * @param uuid The uuid of the volume to retrieve.
     * @return The information of the given volume.
     */
    @GET
    @Path(VOLUME_PATH)
    public Volume getVolume(@PathParam(POOL_PARAM) String poolName,
        @PathParam(VOLUME_PARAM) String uuid);

    /**
     * Creates a new volume.
     * 
     * @param poolName The pool where the volume will be created.
     * @param volume The volume information.
     * @return The information of the created volume.
     */
    @POST
    @Path(VOLUMES_PATH)
    public Volume createVolume(@PathParam(POOL_PARAM) String poolName, Volume volume);

    /**
     * Deletes the given volume.
     * 
     * @param poolName The pool where the volume is.
     * @param uuid The uuid of the volume to delete.
     */
    @DELETE
    @Path(VOLUME_PATH)
    public void deleteVolume(@PathParam(POOL_PARAM) String poolName,
        @PathParam(VOLUME_PARAM) String uuid);

    /**
     * Updates an existing volume.
     * 
     * @param poolName The pool where the volume is.
     * @param volume The new volume information.
     * @return The updated volume.
     */
    @PUT
    @Path(VOLUME_PATH)
    public Volume updateVolume(@PathParam(POOL_PARAM) String poolName, Volume volume);

    // Initiator methods

    /**
     * Allow the provided initiator to access volumes on the provided pool.
     * <p>
     * This method is used to grant access to the given iSCSI initiator to the volume identified by
     * the given UUID.
     * 
     * @param poolName The pool where the volume is.
     * @param ostype, the operating system type of the <code>initiatorIQN</code> (if null or empty
     *            set to <code>default</code>)
     * @param initiatorIQN, client initiator IQN
     * @param volumeUuid The uuid of the volume.
     * @return The iSCSI address where the provided client initiator can reach the volume.
     */
    @POST
    @Path(INITIATOR_PATH)
    public IscsiAddress grantAccess(@PathParam(POOL_PARAM) String poolName,
        @PathParam(OSTYPE_PARAM) String ostype, @PathParam(VOLUME_PARAM) String volumeUuid,
        String initiatorIQN);

    /**
     * Deny the provided initiator to access volumes on the provided pool.
     * <p>
     * This method is used to revoke access for the given iSCSI initiator.
     * 
     * @param pool the pool where the volume is.
     * @param ostype, the operating system type of the <code>initiatorIQN</code> (if null or empty
     *            set to <code>default</code>)
     * @param volumeUuid The uuid of the volume.
     * @param initiatorIQN, client initiator IQN
     */
    @DELETE
    @Path(INITIATOR_PATH)
    public void revokeAccess(@PathParam(POOL_PARAM) String poolName,
        @PathParam(OSTYPE_PARAM) String ostype, @PathParam(VOLUME_PARAM) String volumeUuid,
        @QueryParam(INITIATOR_PARAM) String initiatorIQN);

}
